#include "stm32f0xx_hal.h"
#include "led.h"
#include "button.h"
#include "adc.h"
#include "tim.h"
#include "misc.h"
#include "systick.h"
#include "usart.h"
#include "pwm_out.h"
#include "pwm_in.h"
#include "gpio.h"
#include "logic.h"
#include <stdio.h>


void SystemClock_Config(void);
void Error_Handler(void);




int main()
{
    HAL_Init();
    SystemClock_Config();
     
    gpio_init();
    tim_pwm_out_init();
    tim_pwm_in_init();
    
    usart_init();
    printf("FW started...\r\n");
    
// -----------------------------------------------------------------------------    
// Тесты     
#if 0
    gpio_set_output(true);
    gpio_set_output(false);
    gpio_set_output(true);
    gpio_set_output(false);
    gpio_set_output(true);
    gpio_set_output(false);
    gpio_set_output(true);
    gpio_set_output(false);
    gpio_set_output(true);
    gpio_set_output(false);

#endif    
    
    
    while(1) 
    {
        logic_main();
      
#if 0      
        HAL_Delay(1000);
        tim_print_out_pwm();
        timer_Main();
#endif        
    }
}



void SystemClock_Config(void)
{
    RCC_OscInitTypeDef RCC_OscInitStruct = {0};
    RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
    
    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI|RCC_OSCILLATORTYPE_HSI14;
    RCC_OscInitStruct.HSIState = RCC_HSI_ON;
    RCC_OscInitStruct.HSI14State = RCC_HSI14_ON;
    RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
    RCC_OscInitStruct.HSI14CalibrationValue = 16;
    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
    RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
    RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL12;
    RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
    
    if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
    {
        Error_Handler();
    }
    
    RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                                |RCC_CLOCKTYPE_PCLK1;
    RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
    RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
    RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
    
    if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
    {
        Error_Handler();
    }
}


void Error_Handler(void)
{
    __disable_irq();
    while (1) {}
}
